import sys
import inspect
import re
import os.path
import urllib
import zipfile
import shutil

from django.core.management.base import AppCommand, CommandError
from django.conf import settings
#import ast
from stubtools.core import StubRootCommand
import stubtools

class Command(StubRootCommand):
    help = 'sets-up folder structure for django files and directories if missing from a given project.'
    tab = "    "

    def handle(self, *args, **options):
        super(Command, self).handle()
        self.process()

    def checkInput(self, question, default="n", choices=["y","n"]):

        tags = []

        for x in choices:
            if x == default:
                tags.append(x.upper())
            else:
                tags.append(x)

        tag = "[" + "/".join(tags) + "]"

        result = raw_input(question + " " + tag)
        result = result.lower()

        if result == "":
            result = default

        if result in choices:
            return result

        return self.checkInput(question, default=default, choices=choices)  # Ask the question again if the choice is incorrect

    def process(self):

        result = self.checkInput("Do you want to set-up a split config?")
        if result == "y":
            self.splitConfigSetup()

        result = self.checkInput("Do you want to set-up tempalte paths?")
        if result == "y":
            self.templatePathSetup()

        result = self.checkInput("Do you want to set-up static paths?")
        if result == "y":
            self.staticPathSetup()

        result = self.checkInput("Do you want to get Twitter Bootstrap?")
        if result == "y":
            self.getTwitterBootstrap()

        result = self.checkInput("Do you want to create a Base html template file?")
        if result == "y":
            self.addBaseHtmlFile()

    def splitConfigSetup(self):
        '''
        This sets up the split config that works really well for deployments.  It seperates
        the different deployment options and allows you to override different config settings
        depending on deployment.
        '''
        print "CHECKING FOR DIVIDED SETTINGS"

        project_name = settings.ROOT_URLCONF.split(".")[0]
        settings_file = os.path.join(settings.PROJECT_PATH, project_name, "settings.py")
        if not os.path.isfile( settings_file ):
            print "\tConfig File already Split Up!"
            return
        else:
            print "\tSingle Config File detected, let's break that up."

        settings_dir = os.path.join(settings.PROJECT_PATH, project_name, "settings")

        if not os.path.exists(settings_dir):
            os.makedirs(settings_dir)

        settings_init_file = os.path.join(settings_dir, "__init__.py")

        if not os.path.exists(settings_init_file):  # STUB OUT THE BLANK init FILE
            open(settings_init_file, 'w').close()

        settings_file_c = settings_file + "c"
        if os.path.exists(settings_file_c):
            os.remove(settings_file_c)

        # CHECK FOR THE settings.py AND MOVE IT INTO THE DIRECTORY IF NEEDED
        new_settings_file = os.path.join(settings_dir, "base.py")
        if os.path.exists(settings_file) and not os.path.exists(new_settings_file):
            os.rename(settings_file, new_settings_file)

        # CREATE THE DEV FILE
        dev_settings_file = os.path.join(settings_dir, "dev.py")
        if not os.path.exists(dev_settings_file):
            FILE = open(dev_settings_file, 'w')
            lines = self.loadTemplateLines( os.path.join(stubtools.__path__[0], "templates", "dev_py.txt") )
            FILE.writelines("".join(lines))
            FILE.close()

        # CREATE THE PRODUCTION SERVER FILE
        dev_settings_file = os.path.join(settings_dir, "production.py")
        if not os.path.exists(dev_settings_file):
            FILE = open(dev_settings_file, 'w')
            lines = self.loadTemplateLines( os.path.join(stubtools.__path__[0], "templates", "production_py.txt") )
            FILE.writelines("".join(lines))
            FILE.close()

    def loadTemplateLines(self, path):
            SRC = open(path, 'r')
            lines = SRC.readlines()
            SRC.close()
            return lines

    def templatePathSetup(self):
        print "\nCHECKING FOR TEMPLATE PATH"

        if not len(settings.TEMPLATE_DIRS):
            print "\tERROR: NO TEMPLATE FILE DIRECOTRIES FOUND!\n\tAdd something like this to your config:\n"
            print "TEMPLATE_DIRS = (\n    os.path.join(PROJECT_PATH, 'templates'),\n)\n"
            return

        for directory in settings.TEMPLATE_DIRS:
            if not os.path.exists(directory):
                print "\t\tMAKING %s" % directory
                os.makedirs(directory)
            else:
                print "\tDirectory Exists: %s" % directory

    def staticPathSetup(self):
        print "\nCHECKING FOR STATIC PATHS"

        if not len(settings.STATICFILES_DIRS):
            print "\tERROR: NO STATIC FILE DIRECOTRIES FOUND!\n\tAdd something like this to your config:\n"
            print "STATICFILES_DIRS = (\n    os.path.join(PROJECT_PATH, 'static'),\n)\n"
            return

        if len(settings.STATICFILES_DIRS) == 1:
            for sub in ['css','js','img']:
                directory = os.path.join(settings.STATICFILES_DIRS[0], sub)  
                if not os.path.exists(directory):
                    print "\t\tMAKING %s" % directory
                    os.makedirs(directory)
                else:
                    print "\tDirectory Exists: %s" % directory


    def getTwitterBootstrap(self):
        url = "http://twitter.github.io/bootstrap/assets/bootstrap.zip"

        # SETUP THE DESTINATION PATH
        tmpdir = os.path.join(settings.PROJECT_PATH, "_tmp")
        tmpfile = os.path.join(settings.PROJECT_PATH, "_tmp", url.split("/")[-1])

        # DOWNLOAD
        print "Downloading '%s' \n\tto '%s'" % (url, tmpfile)
        if not os.path.exists(tmpdir):
            os.makedirs(tmpdir)
        urllib.urlretrieve(url, tmpfile)
        print "Download Complete"

        # DECOMPRESS
        try:
            z = zipfile.ZipFile(tmpfile)
        except zipfile.error, e:
            print "Bad zipfile (from %r): %s" % (theurl, e)
            return

        z.extractall(tmpdir)
        z.close()
        os.remove(tmpfile)

        # MOVE TO THE RIGHT LOCATIONS
        bsdir = os.path.join(tmpdir, 'bootstrap')
        bsdirlen = len(bsdir) + 1
        staticdir = settings.STATICFILES_DIRS[0]

        # PREPARE THE LIST
        dir_list = []
        cleanup_list = []

        for (path, dirs, files) in os.walk(bsdir):
            cleanup_list.append(path)
            reldir = path[bsdirlen:]
            for f in files:
                dir_list.append(os.path.join(reldir, f))
        
        # print dir_list

        for item in dir_list:
            os.rename(os.path.join(bsdir, item), os.path.join(staticdir, item))

        # NEED TO ADD CLEANUP OF tmp DIRECTORIES
        shutil.rmtree(tmpdir)

    def addBaseHtmlFile(self):
        pass
        # SETUP THE base.html FILE

        base_html_file = os.path.join(settings.TEMPLATE_DIRS[0], "base.html")
        if not os.path.exists(base_html_file):
            FILE = open(base_html_file, 'w')
            lines = self.loadTemplateLines( os.path.join(stubtools.__path__[0], "templates", "base_html.txt") )
            FILE.writelines("".join(lines))
            FILE.close()






import re, os.path
from django.core.management.base import AppCommand
from django.conf import settings
from stubtools.exceptions import NoProjectPathException
import django

def underscore_camel_case(s):
    """Adds spaces to a camel case string.  Failure to space out string returns the original string.
    >>> space_out_camel_case('DMLSServicesOtherBSTextLLC')
    'DMLS Services Other BS Text LLC'
    """

    return re.sub('((?=[A-Z][a-z])|(?<=[a-z])(?=[A-Z]))', '_', s)


def import_line_check(regex, text, module):
    '''
    Using the given regex see if the module is imported in the text
    '''
    imprt = regex.findall( text )

    if imprt:
        for line in imprt:
            check = [ x.strip() for x in line.split(",") ]
            #print check
            if module in check:
                return True

    return False


def class_name(str):
    return str[:1].upper() + str[1:]


class StubRootCommand(AppCommand):
    """This checks to make sure there is a Project Root path in the Config file"""

    def handle(self, *args, **options):
        try:
            root_path = settings.PROJECT_PATH
        except:
            try:
                root_path = settings.PROJECT_ROOT
            except:
                project_name = settings.ROOT_URLCONF.split(".")[0]

                raise NoProjectPathException(project_name)


def version_check(mode="gte", vcheck="0.0.0"):
    '''
    Checks the production version against the one passed in.  Assumes a
    version number that looks like this: 1.0.0

    5 modes:
        gt -> Greater Than 
        gte -> Greater Than or Equal
        lt ->  Less Than
        lte -> Less Than or Equal
        eq -> Equal
    '''

    django_version = [int(x) for x in django.get_version().split(".")]
    check_version = [int(x) for x in vcheck.split(".")]

    # MAKE SURE THE PASSED IN VERSION HAS 3 VALUES
    if len(check_version) < 3:
        check_version.append(0)

    if len(check_version) < 3:
        check_version.append(0)

    # DETERMINE THE CHECK MODES
    if mode == "gt" and django_version > check_version:
        return True

    if mode == "gte" and django_version >= check_version:
        return True

    if mode == "lte" and django_version <= check_version:
        return True

    if mode == "lt" and django_version < check_version:
        return True

    if mode == "eq" and django_version == check_version:
        return True

    return False




from django.core.management.base import AppCommand, CommandError
import re, os.path
from stubtools.core import underscore_camel_case

class Command(AppCommand):
    args = '<app.model_name>'
    help = 'creates stub model form entries'
    class_regex = re.compile(r"class (\w+)\(.+\):")
    import_line_regex = re.compile(r"^from django import (.+)", re.MULTILINE)
    imports_regex = re.compile(r"(import|from)")
    func_regex = re.compile(r"(def|class)")
    

    def handle(self, *args, **options):
        
        if not args:
            print "No Arguments Passed"
            return
        
        for entry in args:
            app, model = entry.split(".")
            print("CHECKING FOR MODEL FORM FOR: %s" % model)
            self.process(app, model)
        
        
    def process(self, app, model, *args, **kwargs):
        form_file = "%s/forms.py" % app
        print("FORM FILE: %s" % form_file)
        
        import_line = False
        import_entry = False
        first_class_line = 0
        last_import_line = 0
        modelform = model + "Form"
        
        # LOAD FILE
        if os.path.isfile( form_file ): 
            try:
                FILE = open( form_file, "r")
                data = FILE.read()
                
                form_import = self.import_line_regex.findall( data )
                
                if form_import:
                    import_line = True
                    
                    for line in form_import:
                        if 'forms' in [x.strip() for x in line.split(",") ]:
                            import_entry = True
                            
                classes = self.class_regex.findall( data )
                FILE.close()

            except IOError as e:
                pass                    # May need to add something here
                                        # to handle a file locking issue
        # LOOK FOR CLASS WITH NAME
        if modelform in classes:
            print('Form Exists: %s' % modelform)
            return

        print('Creating Form: %s' % modelform)

        import_model_regex = re.compile(r"^from %s.models import (.+)" % (app), re.MULTILINE)
        imported_models = import_model_regex.findall( data )
        
        if not import_entry or imported_models:
            # FIND WHERE TO ADD THE IMPORT LINE
            lines = []
            for m in re.finditer( self.func_regex, data ):
                lines.append( data.count("\n",0,m.start())+1 )
        
            if lines:
                lines.sort()
                first_class_line = lines[0]
        
            print "[%d]" % ( first_class_line )

            lines = []
            for m in re.finditer( self.imports_regex, data ):
                lineno = data.count("\n",0,m.start())+1
                if lineno < first_class_line:
                    lines.append(lineno)
                #print "[%d] %s" % (lineno, data[ m.start() : m.end() ] )
            if lines:
                lines.sort()
                last_import_line = lines[-1]
        
            print "[%d]" % ( last_import_line )
            
        if not import_entry:
            pass
        
        if imported_models:
            imported_models = [x.strip() for x in imported_models[0].split(',')]
        else:
            print "Adding Model with Import Line"
            
        
        #print imported_models
        
        if model not in imported_models:
            print "Adding Model to Import Line"
        
            # CREATE IMPORT LINE
            
            # ADD TO IMPORT LINE
        
        mf = open( form_file, "a" )
        mf.write("\n\nclass %s(forms.ModelForm):\n\n    class Meta:\n        model = %s" % (modelform, model) )
        mf.close()



import sys
import inspect
import re
import os.path
from django.core.management.base import AppCommand, CommandError
from django.conf import settings
#import ast
#app_path = os.path.join(root_path, app_name)

class Command(AppCommand):
    #args = '<app>'
    help = 'Does a simple check pass to see if the project has a few key things set-up.'
    tab = "    "

    def handle(self, *args, **options):

        checklist = []

        checklist.append( self.path_check() )        # Check for the PROJECT_PATH in settings.py

        # SETTING UP THE SPLIT SETTINGS FILES

        # PRINT REPORT
        for check in checklist:
            print "*" * 30
            check.print_result()


    def path_check(self):

        result = Check("PROJECT_PATH Check")

        try:
            root_path = settings.PROJECT_PATH
            result.success = True
        except:
            project_name = settings.ROOT_URLCONF.split(".")[0]
            result.message.append( "You need to have either a PROJECT_PATH settings variable." )
            result.message.append(  "Add the following to your settings.py:\n" )
            result.message.append(  "import os.path\n" )
            result.message.append(  "PROJECT_PATH = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))" )
            result.message.append(  "if '%s/%s' in PROJECT_PATH:" % (project_name, project_name) )
            result.message.append(  "    PROJECT_PATH = PROJECT_PATH.replace('%s/%s', '%s')" % (project_name, project_name, project_name) )
        
        return result


class Check(object):

    def __init__(self, name, message=[], success=False):
        self.name = name
        self.success = success
        self.message = message

    def print_result(self):
        if self.success:
            print "PASS: %s" % self.name
        else:
            print "FAIL: %s" % self.name

        if self.message:
            print '\n'.join(self.message)





import sys
import inspect
import re
import os.path
from django.core.management.base import AppCommand, CommandError
from django.conf import settings
#import ast

class Command(AppCommand):
    args = '<app>'
    help = 'creates boilerplate django files if missing from a given app.'
    tab = "    "

    def handle(self, *args, **options):
        if len(args) < 1:
            raise CommandError('Need to pass App names')

        app_name = args[0]
        try:
            root_path = settings.PROJECT_PATH
        except:
            try:
                root_path = settings.PROJECT_ROOT
            except:
                project_name = settings.ROOT_URLCONF.split(".")[0]

                print "You need to have either a PROJECT_PATH or PROJECT_ROOT settings variable."
                print "Add the following to your settings.py:\n"
                print "import os.path\n"
                print "PROJECT_ROOT = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))"
                print "if '%s/%s' in PROJECT_ROOT:" % (project_name, project_name)
                print "    PROJECT_ROOT = PROJECT_ROOT.replace('%s/%s', '%s')" % (project_name, project_name, project_name)
                return

        app_path = os.path.join(root_path, app_name)
        self.add_url(app_name, app_path)
        self.add_admin(app_name, app_path)
        self.add_forms(app_name, app_path)

    def add_url(self, app_name, app_path):
        file_name = 'urls.py'
        comment = "#"
        lines = []

        # SHOULD LOOK AT CLASS BASED VIEWS IN views.py FOR URL GENERATION

        lines.append("from django.conf.urls.defaults import patterns, url, include")
        lines.append("from %s import views\n" % app_name)
        lines.append("urlpatterns = patterns(\"\",")
        lines.append("%s%surl(regex=r'^$', view=views.SomeView.as_view(), name='%s_view'),\n)" % (comment, self.tab, app_name) )
        lines.append("\n\n")

        if self.add_file(app_name, app_path, file_name, '\n'.join(lines)):
            print "Added URLs: %s" % app_name

    def add_admin(self, app_name, app_path):

        file_name = 'admin.py'

        lines = []
        comment, models = self.get_models(app_name, app_path)

        lines.append("from django.contrib import admin")
        lines.append("%sfrom %s.models import %s\n" % (comment, app_name, ", ".join(models)))

        for model in models:
            lines.append("%sadmin.site.register(%s)\n" % (comment, model))
        
        lines.append("\n\n")

        if self.add_file(app_name, app_path, file_name, '\n'.join(lines)):
            print "Added admin.py: %s" % app_name

    def add_forms(self, app_name, app_path):

        file_name = 'forms.py'

        #comment = ""
        lines = []
        comment, models = self.get_models(app_name, app_path)

        if not models:
            print "No Models to create Forms from."
            return

        if models[0] == "ExampleModel":
            comment = "#"

        lines.append("from django import forms")
        lines.append("%sfrom %s.models import %s\n" % (comment, app_name, ", ".join(models)))

        for model in models:
            lines.append("%sclass %sForm(forms.ModelForm):" % (comment, model))
            lines.append(comment)
            lines.append(comment + self.tab + "class Meta:")
            lines.append(comment + (2 * self.tab) + "model = %s\n\n" % (model))
        
        lines.append("\n\n")

        if self.add_file(app_name, app_path, file_name, '\n'.join(lines)):
            print "Added forms.py: %s" % app_name

    def add_file(self, app_name, app_path, file_name, lines):
        file_path = os.path.join(app_path, file_name)

        if os.path.isfile(file_path):
            print "Path exists, skipping: %s" % file_path
            return False

        try:
            FILE = open(file_path, "w")
            FILE.write( lines )
            FILE.close()
        except IOError as e:
            print( "IO Error reading %s\n\t%s" % (file_path, e) )
            return

        return True

    def get_models(self, app_name, app_path):

        models = []
        comment = ""

        try:
            classes = inspect.getmembers(sys.modules["%s.models" % app_name], inspect.isclass)  # COULD ALSO ADD CHECK TO SEE IF IT IS A DJANO MODEL CLASS
        except KeyError:
            print "\'%s\'' app is not included in your installed apps, please include it and run again." % app_name
            return comment, models

        for cl in classes:
            if cl[1].__module__ == "%s.models" % app_name:
                models.append(cl[0])

        if not models:
            models = ['ExampleModel']
            comment = "#"

        return comment, models





from django.core.management.base import AppCommand, CommandError
from stubtools.core import class_name, version_check
import re, os.path
import ast
import django

class Command(AppCommand):
    args = '<app page_name>'
    help = 'creates a template and matching method for a given page name'

    def handle(self, *args, **options):
        if len(args) < 1:
            raise CommandError('Need to pass App.Page names')
            
        parts = args[0].split(".")

        # Using a Dictionary for clarity in strin replacements
        argDict = {'app':parts[0], 'page':parts[1].lower()}
        tab = "\t"
        argDict['tab'] = " " * 4
        
        use_class_based_views = version_check("gte", "1.3.0")        # SHOULD DEFAULT FOR 1.3+ TO True.  NEED ATTR IN settings.py TO CONFIG SET TO FALSE.
        
        #from django.views.generic import TemplateView
        view_file = "%(app)s/views.py" % argDict
        argDict['view_file'] = view_file

        views = []

        if use_class_based_views:
            url_entry_regex = re.compile("url\(\S+ (\S+)" )
        else:
            url_entry_regex = re.compile("url\(\S+ '(\S+)'" )
        
        # Get contents of views.py file
        if os.path.isfile(view_file): 
            try:
                FILE = open(view_file, "r")
                data = FILE.read()
                FILE.close()
            except IOError as e:
                print( "IO Error reading %s\n\t%s" % (view_file, tab, e) )
                return
        
        insert_line = None
        replace_line = None
        
        if use_class_based_views:
            views = [ x.name for x in ast.parse(data).body if isinstance(x, ast.ClassDef) ]   # BEST PYTHON WAY TO DO THIS
            view_name = ( "%sView" % ( class_name( argDict['page'] ) ) )
            view_import_path = "views.%s.as_view()" % (view_name)
            argDict['view_import_path'] = view_import_path
            
            # CHECK IMPORT LINES
            importers = { v.module : v for v in ast.parse(data).body if isinstance(v, ast.ImportFrom) }
            
            if not importers:
                insert_line = (0, "from django.views.generic import TemplateView\n\n")
            else:
                if "django.views.generic" in importers:
                    name_list = [ x.name for x in importers["django.views.generic"].names ]
                    
                    if "TemplateView" not in name_list:
                        print("Adde Module into line: %d -> %s" % (importers["django.views.generic"].lineno, "TemplateView" ) )     # NEED AUTOMATIC WAY TO INSERT THIS
                else:
                    # GET THE LAST IMPORT LINE
                    import_number = 0
                    insert_line = (import_number, "from django.views.generic import TemplateView\n")
            
        else:
            views = [ x.name for x in ast.parse(data).body if isinstance(x, ast.FunctionDef) ]   # BEST PYTHON WAY TO DO THIS
            view_name = ("%(page)s_view" % argDict)
            view_import_path = "%(app)s.views.%(page)s_view" % argDict
            argDict['view_import_path'] = view_import_path
            
        url_name = "%(app)s-%(page)s" % argDict
        argDict['url_name'] = url_name
        template = "%(app)s/%(page)s.html" % argDict
        
        FILE = open(view_file, "r")
        lines = FILE.readlines()
        FILE.close()
        #lines = [ x.strip() for x in FILE.readlines() ]    # WILL STRIP OFF TABS
        dirty = False

        if insert_line:
            lines.insert( insert_line[0], insert_line[1] )
            dirty = True
        
        # If the new page name is not in the views.py, add the stub
        if view_name not in views:
            self.stdout.write( "ADDING: %s to %s\n" % (argDict['page'], view_file) )
            dirty = True

            if use_class_based_views:
                # CHECK FOR IMPORT LINE IN FILE

                lines.extend( [ "class %s(TemplateView):\n" % view_name, 
                                argDict['tab'] + "template_name = '%s'\n\n\n" % template ] )
            else:
                lines.extend( [ "def %s(request):\n" % view_name, 
                                argDict['tab'] + "ctx = RequestContext(request)\n",
                                argDict['tab'] + "return render_to_response('%s', ctx )\n" % template ] )
            
        else:
            self.stdout.write( "EXISTING VIEWS: %s\n" % ", ".join(views) )

        if dirty:
            FILE = open(view_file, "w")
            FILE.writelines( lines )
            FILE.close()

        # Create the template stub
        template_file = "templates/%s" % template
        
        # Need to check for directory and add it if it is missing from the template directory
        if not os.path.isfile(template_file): 
            
            dest_path = "templates/%(app)s" % argDict
            if not os.path.exists(dest_path):
                os.makedirs(dest_path)

            self.stdout.write( "ADDING TEMPLATE FILE: %s\n" % template_file )
            FILE = open( template_file, "w" )
            html = ['<!DOCTYPE html>',
                    '<html>',
                    '<head>',
                    '<meta charset="utf-8">',
                    '\t<title>%(app)s - %(page)s</title>' % argDict,
                    '</head>',
                    '<body>',
                    '\t<h1>%(app)s - %(page)s, Stub Page</h1>' % argDict,
                    '\t<a href="{% url ' + "'%(url_name)s'" % argDict + ' %}">link</a>',
                    '</body>\n</html>']

            FILE.write( "\n".join(html) )

        url_file = "%(app)s/urls.py" % argDict

        #################
        # UPDATE THE urls.py FILE

        argDict['page_link'] = "%(page)s/" % argDict

        if argDict['page'] == "index":
            argDict['page_link'] = "/"

        if os.path.isfile(url_file):
            '''
            If there is a urls.py file do this
            '''
            try:
                FILE = open(url_file, "r")
                data = FILE.read()
                urls = url_entry_regex.findall( data )
                FILE.close()

            except IOError as e:
                print( "IO Error reading %s, Step Skipped.\n\t%s" % (view_file, e) )
            
            if argDict['view_import_path'] + "," not in urls:   # Make sure to add the comma, which is caught by the regex pattern
                FILE = open(url_file, "a")

                if use_class_based_views:
                    url_py = "urlpatterns += patterns('',\n%(tab)surl(r'^%(page_link)s$', %(view_import_path)s, name='%(url_name)s'),\n)" % (argDict)
                else:
                    url_py = "urlpatterns += patterns('',\n%(tab)surl(r'^%(page_link)s$', '%(view_import_path)s', name='%(url_name)s'),\n)" % (argDict)

                FILE.write( url_py + "\n" )
                FILE.close()

        else:
            '''
            If there is no urls.py file do this
            '''
            FILE = open(url_file, "w")

            if version_check("gte", "1.5.0"):
                url_py = ["from django.conf.urls import *", "from %(app)s import views\n" % argDict, "urlpatterns = patterns(''," ]
            else:
                url_py = ["from django.conf.urls.defaults import *\n", "urlpatterns = patterns(''," ]

            if use_class_based_views:
                url_py.append(   "%(tab)surl(r'^%(page_link)s$', %(view_import_path)s, name='%(url_name)s'),\n)" % ( argDict ) )
            else:
                url_py.append(   "%(tab)surl(r'^%(page_link)s$', '%(view_import_path)s', name='%(url_name)s'),\n)" % ( argDict ) )

            FILE.write( "\n".join(url_py) + "\n" )
            FILE.close()


    def print_break(self, lable):
        count = 20
        self.stdout.write( "\n" + "*" * count + "\n" )
        self.stdout.write( "%s\n" % lable )
        self.stdout.write( "*" * count + "\n" )





from django.core.management.base import AppCommand, CommandError
import re, os.path
from stubtools.core import underscore_camel_case, import_line_check, class_name


class Command(AppCommand):
    args = '<app.model_name>'
    help = 'creates stub Templates, Forms and Admin entries for a given model name'
    import_line_regex = re.compile(r"^from django.db import (.+)", re.MULTILINE)
    import_uget_regex = re.compile(r"^from django.utils.translation import (.+)", re.MULTILINE)
    imports_regex = re.compile(r"(import|from)")
    class_regex = re.compile(r"class (\w+)\(.+\):")
    func_regex = re.compile(r"(def|class)")
    

    def handle(self, *args, **options):
        
        if not args:
            print "No Arguments Passed"
            return
        
        for entry in args:
            app, model = entry.split(".")
            print("CHECKING FOR MODEL: %s" % model)
            self.process(app, model)
        
        
    def process(self, app, model, *args, **kwargs):
        model_file = "%s/models.py" % app
        print("MODEL FILE: %s" % model_file)
        
        import_entry = False
        first_class_line = 0
        last_import_line = 0
        new_lines = []

        # MAKE SURE THE MODEL NAME'S FIRST LETTER IS CAPITALIZED
        model = class_name(model)
        
        # LOAD FILE
        if os.path.isfile( model_file ): 
            try:
                FILE = open( model_file, "r")
                data = FILE.read()

                if not import_line_check(self.import_line_regex, data, 'models'):
                    new_lines.append("from django.db import models")

                if not import_line_check(self.import_uget_regex, data, 'ugettext_lazy as _'):
                    new_lines.append("from django.utils.translation import ugettext_lazy as _")

                classes = self.class_regex.findall( data )
                FILE.close()

            except IOError as e:
                print "IO ERROR, CONTINUE"
                pass                    # May need to add something here
                                        # to handle a file locking issue
        else:
            print( "File Not Found: %s" % model_file )
            return

        # LOOK FOR CLASS WITH NAME
        if model in classes:
            print('Model Exists: %s' % model)
            return

        print('Creating Model: %s' % model)
        
        if not import_entry:
            # FIND WHERE TO ADD THE IMPORT LINES
            
            lines = []
            for m in re.finditer( self.func_regex, data ):
                lines.append( data.count("\n",0,m.start())+1 )
        
            if lines:
                lines.sort()
                first_class_line = lines[0]
        
            print "[%d]" % ( first_class_line )

            lines = []
            for m in re.finditer( self.imports_regex, data ):
                lineno = data.count("\n",0,m.start())+1
                if lineno < first_class_line:
                    lines.append(lineno)
                #print "[%d] %s" % (lineno, data[ m.start() : m.end() ] )
            if lines:
                lines.sort()
                last_import_line = lines[-1]
        
            print "[%d]" % ( last_import_line )
        
        # ADD THE MODEL TO THE LINES
        new_lines.append('\n\nclass %s(models.Model):\n    name = models.CharField(_("Name"), max_length=300)' % model)
        new_lines.append(" ")

        mf = open( model_file, "a" )
        # NEEDS TO LOAD AND REWRITE THE FILE RATHER THAN JUST APPEND
        mf.write( "\n".join(new_lines) )
        mf.close()



import sys
import inspect
import re
import os.path
from django.core.management.base import CommandError
from django.conf import settings
#import ast
from stubtools.core import StubRootCommand

class Command(StubRootCommand):
    #args = '<app>'
    help = 'sets-up boilerplate django files and directories if missing from a given project.'
    tab = "    "

    def handle(self, *args, **options):
        super(Command, self).handle()

        if len(args) < 1:
            raise CommandError('Need to pass App names')

        app_name = args[0]
        app_path = os.path.join(root_path, app_name)


    def get_models(self, app_name, app_path):

        models = []
        comment = ""

        try:
            classes = inspect.getmembers(sys.modules["%s.models" % app_name], inspect.isclass)  # COULD ALSO ADD CHECK TO SEE IF IT IS A DJANO MODEL CLASS
        except KeyError:
            print "\'%s\'' app is not included in your installed apps, please include it and run again." % app_name
            return comment, models

        for cl in classes:
            if cl[1].__module__ == "%s.models" % app_name:
                models.append(cl[0])

        if not models:
            models = ['ExampleModel']
            comment = "#"

        return comment, models





from distutils.core import setup

setup(name='django-stubtools',
      version='0.4.7',
      author = "Grant Viklund",
      author_email = "gviklund@backcode.com",
      description = "A set of tools for Django to help 'stub-out' an app quickly.",
      license = "BSD",
      keywords = "django app",
      home_page = "http://github.com/backcode/django-stubtools",
      packages=['stubtools', 'stubtools.core', 'stubtools.management', 'stubtools.management.commands', 'stubtools.tests', 'stubtools.templates'],
      package_data={'stubtools.templates': ['*.txt']},
      classifiers=[
          "Development Status :: 4 - Beta",
          "Framework :: Django",
          "Intended Audience :: Developers",
          "License :: OSI Approved :: BSD License",
      ],
      install_requires=[
          "Django >= 1.4.0",
      ],
)



# EXCEPTIONS

class NoProjectPathException(Exception):
    def __init__(self, message):

    	project_name = message
    	result = []

        result.append( "You need to have a PROJECT_PATH settings variable." )
        result.append( "Add the following to your settings.py:\n" )
        result.append( "import os.path\n" )
        result.append( "PROJECT_PATH = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))" )
        result.append( "if '%s/%s' in PROJECT_PATH:" % (project_name, project_name) )
        result.append( "    PROJECT_PATH = PROJECT_PATH.replace('%s/%s', '%s')" % (project_name, project_name, project_name) )

        #result.append( message )

        # Call the base class constructor with the parameters it needs
        Exception.__init__(self, '\n'.join(result) )

        # Now for your custom code...
        #self.Errors = Errors

        # print "You need to have either a PROJECT_PATH or PROJECT_ROOT settings variable."
        # print "Add the following to your settings.py:\n"
        # print "import os.path\n"
        # print "PROJECT_ROOT = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))"
        # print "if '%s/%s' in PROJECT_ROOT:" % (project_name, project_name)

        # print "    PROJECT_ROOT = PROJECT_ROOT.replace('%s/%s', '%s')" % (project_name, project_name, project_name)
        # raise Exception("Missing ")
